/*
NAME: 
DESCRIPTION: 负片散热pad制作;
PARAMETER:
    [
		{
			name : 'step',
			title : 'Step过滤',
			type : 'LineEdit',
			property : {tool_tip : '支持正则'},
		},
		{
            name : 'auto_save',
			title : '自动保存',
            type : 'RadioBox',
            property : {
				item_list:[
					{name:'yes',text:'YES'},
					{name:'no',text:'NO'},
				],
				tool_tip:'是否自动保存料号开关'
			}
        }
	]
	
 VERSION_HISTORY:
	V1.00 2020-04-28 Scott Sun
	    1.新版本
		
 HELP:
 	<html><body bgcolor="#DDECFE">
		<font size="3" color="#003DB2"><p>功能简介</p></font>
		<p> 散热pad </p>
		<br>
		<font size="3" color="#003DB2"><p>参数配置</p></font>
		<p> 配置 </p>
		<br>
		<font size="3" color="#003DB2"><p>注意事项</p></font>
		<p> 无 </p>
		<br>
	</body></html>	
*/
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
console.log("==============================>JTG_INN_neg_thermal_pad_optimization_aim");
/* question
*/
// 引入模块 包
var $ = require('topcam.scriptfunc').argv();
var fs = require('fs');
var _ = require('lodash');
var mode = $.ikm ? "topcam" : "aimdfm";
var IKM = $.ikm; 
if (IKM==undefined ) { IKM = require('topcam.ikm6')($) }
var GEN = $.gen;
var GUI = $.gui || {};
var Job = $.job || $.job_name;
var JobId = $.job_id;
var db = $.db || IKM.db;
var PAR = {}
if ($.par) {
	PAR = $.par
} else if ($.hasOwnProperty('script_parameter')){
	PAR = JSON.parse($.script_parameter);
}
if (mode === "aimdfm") {
	var database = require("topsin.database");
	database.addConnection($.conf.database_conf, "DFM");
	var QDfm = database.query("DFM");
	$.QDfm = QDfm;
	if ($.conf.product_type == "aimdfm") {
		QDfm.updateRow({
			table: "pdm_aimdfm_task",
			data: {
				current_process_title: $.process_title
			},
			where: { id: $.task_id }
		});
	}
}
var Status = 'ok';
var resultData = [];

try {
    var par = PAR;
    var default_par = {  // par默认参数？
		step:"edit",
		units: "mm",
		auto_save: "no"
	}
	for(var key in default_par){  // 设置默认属性
		if (!par.hasOwnProperty(key) || par[key] == ""){
			par[key] = default_par[key]
		}
	}

	if(_.isEmpty(Job)){throw "参数Job不存在"}
    var job = Job.toLowerCase()
    // 料号验证
	if(!GEN.isJobExists({job:job})){throw "料号"+job+"不存在"}
	if(!GEN.isJobOpen({job:job})){ GEN.openJob({job:job}) }
	if(mode == "aimdfm"){
        if(GEN.checkInout({job:job,mode:"test"}) != 0){ throw "the job check" }
        GEN.checkInout({job:job,mode:"out"});
    }
	
	// 脚本开始
	// 通过matrix选出属性为negative&power_ground&inner的层 context为board的
	var matrix = GEN.getMatrix({job:job})
	var n_layers = Object.keys(matrix).sort(function(a,b){return matrix[a].row - matrix[b].row})
	.filter(function(key){
		var val = matrix[key]
		return val.layer_type == "power_ground" && val.context == "board" && val.polarity == "negative" && val.side == "inner"
	})
	if(n_layers.length != 0){
	var stepList = GEN.getStepList({job:job})
	if(par.step != ""){
		stepList = stepList.filter(function(v){
			var step_reg = new RegExp(par.step)
			return step_reg.test(v)})
	}
	
	var tmp1 = "tmp_1", tmp2 = "tmp_2"
	var tmp_layers = [tmp1, tmp2];
	stepList.forEach(function(step){
		GEN.openStep({job:job, name:step})
		GEN.units({type:"inch"})
		GEN.affectedLayer({affected:'no',mode:'all'})
		GEN.clearLayers()
		tmp_layers.forEach(function(v){  // 如果辅助层存在先删除
			if(GEN.isLayerExists({job:job, layer: v})){GEN.deleteLayer({job:job, layer: v})}
		})
		GEN.affectedLayer({affected:'no',mode:'all'})
		n_layers.forEach(function(layer){
			// 逐层选择出上th*，i274*属性的物件并移动到辅助层；
			GEN.workLayer({name:layer,display_number:2,clear_before:'yes'})
			tmp_layers.forEach(function(v){
				if(GEN.isLayerExists({job:job,layer:v})){GEN.deleteLayer({job:job, layer:v})}
				GEN.createLayer({job:job, layer:v})
			})
			GEN.selectByFilter({include_syms:"th*;i274*"})
			if(GEN.getSelectCount()>0){
				// 移动到辅助层
				GEN.COM("sel_move_other",{target_layer : tmp1,invert : 'no',dx : '0',dy : '0',size : '0'});
				GEN.workLayer({name:tmp1,display_number:2,clear_before:'yes'})
				GEN.COM("sel_change_sym",{symbol : "r1.1",reset_angle : 'no'});
				// 以辅助层为参照选择出drl层上有散热盘的孔，将这些孔copy辅助层2
				GEN.workLayer({name:"drl",display_number:2,clear_before:'yes'})
				GEN.selClearFeature()
				GEN.selRefFeat({layers:tmp1,use:'filter',mode:'touch'})
				if(GEN.getSelectCount()>0){
					GEN.COM("sel_copy_other",{dest:"layer_name",target_layer:tmp2,invert:"no",dx:0,dy:0,size:0,x_anchor:0,y_anchor:0,rotation:0,mirror:"none"})
					// 获取辅助层2上的pad大小，依次计算不同pad对应散热盘尺寸，圆孔（pad）thr*，槽孔(line)（rc_ths*）；
					GEN.workLayer({name:tmp2,display_number:2,clear_before:'yes'})
    				var syms = GEN.getLayerSymsHist({job:job,step:step,layer:tmp2,units:"inch"})
					console.log(_.toString(syms))
					for (var key in syms) {
						var new_thr = "thr"
						var op = "20"
						// 圆孔（pad）thr*
						if(/r\d+\.\d/.test(key)){
							var tmp = /r(\d+)\.(\d)/.exec(key)
							var i1 = tmp[1]
							var i2 = tmp[2]
							if(i1 <= 20){op = '15';}
							if(i2 >= 5){new_thr = 'thr'+String(Number(i1) + 41)+'x'+String(Number(i1) + 21)+'x45x4x'+op}
							if(i2 < 5){new_thr = 'thr'+String(Number(i1) + 40)+'x'+String(Number(i1) + 20)+'x45x4x'+op}
							GEN.COM("sel_multi_feat",{operation : 'select',feat_types : 'pad', include_syms : key});
							GEN.COM("sel_change_sym",{symbol : new_thr,reset_angle : 'no'});
							GEN.COM("sel_multi_feat",{operation : 'select',feat_types : 'pad', include_syms : new_thr});
							GEN.COM("sel_move_other",{target_layer : layer,invert : 'no',dx : '0',dy : '0',size : '0'});
						}
						if(/^r\d+$/.test(key)){
							var i = /^r(\d+)$/.exec(key)[1]
							if(i <= 20) {op = "15"}
							new_thr = 'thr'+String(Number(i) + 40)+'x'+String(Number(i) + 20)+'x45x4x'+op
							GEN.COM("sel_multi_feat",{operation : 'select',feat_types : 'pad', include_syms : key});
							console.log(new_thr)
							GEN.COM("sel_change_sym",{symbol : new_thr,reset_angle : 'no'});
							GEN.COM("sel_multi_feat",{operation : 'select',feat_types : 'pad', include_syms : new_thr});
							GEN.COM("sel_move_other",{target_layer : layer,invert : 'no',dx : '0',dy : '0',size : '0'});
						}
					}
				}
			}
		})
	})
	GEN.deleteLayer({job:job, layer:tmp_layers})
	}
	// 保存 
	if(/yes/ig.test(par.auto_save)){
		GEN.checkInout({job:job,mode:"out"})  // 结束保存料号 关闭料号
		GEN.saveJob({ job: job });
		GEN.checkInout({job:job,mode:"in"})
		GEN.closeJob({job:job})
	} else {
		GEN.checkInout({job:job,mode:"in"})
	}
    console.log("======================== JTG_INN_neg_thermal_pad_optimization_aim end ");
	if (mode === "aimdfm") {
		$.QDfm.updateRow({
			table: "pdm_aimdfm_task",
			data: {
				progress: 33.33
			},
			where: { id: $.task_id }
		});
		if (GEN.hasError()) {
			Status = 'error';
			resultData.push({ type: "error", title: "GEN错误!", detail: [{ desc: _.join(GEN.STATUS, "\n") }] });
			return {
				status: Status,
				result_data: resultData
			};
		} else {
			resultData.push({ type: "info", title: "操作完成, 请注意检查!" });
			return {
				status: Status,
				result_data: resultData
			};
		}
	}else {
		return "Done"
	}
} catch (e) {
	IKM.msg(_.join(GEN.STATUS, "\n"))
	IKM.msg(e)
    Status = 'error';
    resultData.push({type: "error", title: "脚本执行出错!", detail: [{desc: _.toString(e)}]});
	return (mode === "aimdfm") ? {status: Status, result_data: resultData} : "Error";
}