/*
 NAME: 
 DESCRIPTION: 上传matrix信息/存储profile信息
 PARAMETER:
	[
		{
			name : 'mode',
			type : 'MultiComboBox',
			title : '工作模式',
			property : {
				tool_tip : '可根据实际需要，单选或多选,未设定，则提醒必须设定',
				size_policy:'Expanding,Fixed',
				item_list:[
					{name:'upload_profile',text:'上传profile信息'},
					{name:'upload_matrix',text:'上传matrix信息'},
					{name:'upload_copper_area',text:'上传残铜信息'},
				]
			},
			pack : {row:1,column:1},
		},
		{
			name : 'pcs_step',
			type : 'LineEdit',
			title : 'Pcs Step过滤',
			pack : {row:0,column:1},
			property:{tool_tip:'上传profile信息时，必须定义,否则会提醒'},
		},
		{
			name : 'array_step',
			type : 'LineEdit',
			title : 'Array Step过滤',
			pack : {row:0,column:1},
			property:{tool_tip:'上传profile信息时，必须定义,否则会提醒'},
		},
		{
			name : 'coupon_step',
			type : 'LineEdit',
			title : 'Coupon Step过滤',
			pack : {row:0,column:1},
			property:{tool_tip:'上传profile信息时，有定义且有step存在，则会存储，否则不存储'},
		},
		{
			name : 'other_step',
			type : 'LineEdit',
			title : '其他Step过滤',
			pack : {row:0,column:1},
			property:{tool_tip:'上传profile信息时，有定义且有step存在，则会存储，否则不存储'},
		},
		{
			name : 'units',
			title : 'Units',
			type : 'ComboBox',
			property : {
				size_policy:'Expanding,Fixed',
				item_list:[
					{name:'mm',text:'mm'},
					{name:'inch',text:'inch'},
				],
				tool_tip:'上传profile信息时的单位，如未设置，默认为inch'
			},
			pack : {row:1,column:1},
		},
		{
			name : 'matrix_type',
			type : 'MultiComboBox',
			title : 'matrix信息类型',
			property : {
				tool_tip : '可根据实际需要，单选或多选,未设定，则默认为工作稿',
				size_policy:'Expanding,Fixed',
				item_list:[
					{name:'org',text:'原稿'},
					{name:'work',text:'工作稿'},
				]
			},
			pack : {row:1,column:1},
		},
	]
	
	
 VERSION_HISTORY:
	V1.00 2020-06-01 Carver
		1.新版本
		
 HELP:
	<html><body bgcolor="#DDECFE">
		<font size="3" color="#003DB2"><p>功能简介</p></font>
		  <p> 上传matrix信息 </p>
		  <br>
		<font size="3" color="#003DB2"><p>注意事项</p></font>
		  <p> ● 无 </p>
	</body></html>
*/
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
console.log("============================>JTG_PUB_upload_matrix_and_profile start");
// 引入模块 包
var $ = require('topcam.scriptfunc').argv();
var _ = require('lodash');
var Error = require("topsin.error");

var mode = $.ikm ? "topcam" : "aimdfm";
var IKM = $.ikm; 
if (IKM==undefined ) { IKM = require('topcam.ikm6')($) }
var GEN = $.gen;
var GUI = $.gui || {};
var Job = $.job || $.job_name;
var JobId = $.job_id;
var db = $.db || IKM.db
var PAR = {}
if ($.par) {
	PAR = $.par
} else if ($.hasOwnProperty('script_parameter')){
	PAR = JSON.parse($.script_parameter);
}
if (mode === "aimdfm") {
	var database = require("topsin.database");
	database.addConnection($.conf.database_conf, "DFM");
	var QDfm = database.query("DFM");
	$.QDfm = QDfm;
	if ($.conf.product_type == "aimdfm") {
		QDfm.updateRow({
			table: "pdm_aimdfm_task",
			data: {
				current_process_title: $.process_title
			},
			where: { id: $.task_id }
		});
	}
}
var Status = 'ok';
var Return = 'finish';
var resultData = [];

if(!PAR.mode){
	// GUI.msgBox('error', 'error', '脚本参数中【工作模式】必须定义, 请确认!');
	// return 'Error';

	PAR.mode = 'upload_matrix';  // 默认是 upload_matrix 
}

if(/upload_profile/.test(PAR.mode)){
    if(!PAR.units){
        PAR.units = 'inch';
    }
}

if(/upload_matrix/.test(PAR.mode)){
	// if(!PAR.public_script){
	// 	GUI.msgBox("error", "error", "脚本参数中【公用脚本名称】必须定义, 请确认!");  // 公共脚本此处不需要管
	// 	return 'Error';
	// }
	
    if(!PAR.matrix_type){
        PAR.matrix_type = 'work';
    }
}

PAR.pcs_step = PAR.pcs_step || 'pcs';
PAR.array_step = PAR.array_step || 'array';
PAR.coupon_step = PAR.coupon_step || 'coupon';

try {
    // 检查Genesis料号是否存在并打开
	if(!GEN.isJobExists({job:Job})){
		GUI.msgBox('error', 'error', '料号'+Job+'在Genesis中不存在, 请检查!');
	}

	if(!GEN.isJobOpen({job:Job})){
		GEN.openJob({job:Job});
	}

	// 残铜率
	if(/upload_copper_area/.test(PAR.mode)){
		// 有array计算array，否则计算pcs
		if(GEN.isStepExists({job:Job,step:PAR.array_step})){
			GEN.openStep({job:Job, name:PAR.array_step});
		}else if(GEN.isStepExists({job:Job,step:PAR.pcs_step})){
			GEN.openStep({job:Job, name:PAR.pcs_step});
		}else{
			throw 'array and pcs none exist!'; 
		}

		var matrix = GEN.getMatrix({job:Job})
		// IKM.msg(_.toString(matrix)) //

		var signal_layer = Object.keys(matrix).reduce(function(a, b){  // matrix过滤出线路层
			if(matrix[b].context == "board"){  // power_ground是负层
				if(matrix[b].layer_type == "signal" || matrix[b].layer_type == "power_ground"){
					a.push(b)
				}            
			}
			return a;
		},[])
		console.log('===>signal_layer: ', _.toString(signal_layer))

		signal_layer.forEach(function(layer){  // 得出所有线路层的残铜率
			res =  GEN.copperArea({layer1:layer})
			if(res.percent != ''){
				IKM.save_layerinfo({  // 最终是存储在表 pdm_job_layerattr 中
					'jobid': JobId,
					'layer': layer,
					'layerinfohash': {
						// 'copper_area_ratio': res.percent + '%', //不加 %

						// 打开料号 => 料号叠构 => 新建叠层 => 查看铜分布率 验证是否存入。但是此处修改数据后再次存入的表不一样。
						'copper_area_ratio': res.percent,  
					}
				});
			}
		})

		GEN.closeStep();
	}

	// ##
	if( /upload_profile/.test(PAR.mode) && (PAR.pcs_step || PAR.array_step || PAR.coupon_step || PAR.other_step) ){
		var step_list = GEN.getStepList({job:Job})

		// ##
		var pcs_step = [];
		if(PAR.pcs_step){
			var reg = new RegExp("^"+PAR.pcs_step, "g");
			pcs_step = step_list.reduce(function(a, b){
				if(reg.test(b)){
					a.push(b);
				}

				return a;
			}, [])
		}

		// ##
		var array_step = [];
		if(PAR.array_step ){
			var reg = new RegExp("^"+PAR.array_step, "g");
			array_step =step_list.reduce(function(a, b){
				if(reg.test(b)){
					a.push(b);
				}

				return a;
			}, [])
		}

		if(array_step.length > 0){
			array_step.forEach(function(arr){
				var ans = save_profile_info({step:arr,step_type:'array'});
				if(ans){
					GUI.msgBox("warning", "warning", _.toString(ans));
					return 'Error';
				}
			})
		}else{
			if(pcs_step.length > 0){	
				pcs_step.forEach(function(pcs){
					var ans = save_profile_info({step:pcs,step_type:'array'});
					if(ans){
						GUI.msgBox("warning", "warning", _.toString(ans));
						return 'Error';
					}
				})
			}
		}

		// ##
		if(PAR.coupon_step){
			var reg = new RegExp("^"+PAR.coupon_step, "g");
			var coupon_step =step_list.reduce(function(a, b){
				if(reg.test(b)){
					a.push(b);
				}

				return a;
			}, [])

			coupon_step.forEach(function(pcs){
				var ans = save_profile_info({step:pcs,step_type:'coupon'});
				if(ans){
					GUI.msgBox("warning", "warning", _.toString(ans));
					return 'Error';
				}
			})
		}

		// ##
		if(PAR.other_step){
			var reg = new RegExp("^"+PAR.other_step, "g");
			var other_step =step_list.reduce(function(a, b){
				if(reg.test(b)){
					a.push(b);
				}

				return a;
			}, [])

			other_step.forEach(function(pcs){
				var ans = save_profile_info({step:pcs,step_type:pcs});
				if(ans){
					GUI.msgBox("warning", "warning", _.toString(ans));
					return 'Error';
				}
			})
		}
	}

	// ##
	if(/upload_matrix/.test(PAR.mode)){
		// doScript(PAR.public_script);

		if(/org/.test(PAR.matrix_type)){
			var ans = UPLOAD_LAYER_MATRIX({job:Job, jobcategory:'org'});
			if (!ans){  // 
				GUI.msgBox("warning", "warning", _.toString(ans));
				return 'Error';
			}

			ans = UPLOAD_LAYER_MATRIX({job:Job,jobcategory:'work'});
			if (!ans){
				GUI.msgBox("warning", "warning", _.toString(ans));
				return 'Error';
			}
		}
		if(/work/.test(PAR.matrix_type)){
			var ans = UPLOAD_LAYER_MATRIX({job:Job, jobcategory:'work'});
			if (!ans){
				GUI.msgBox("warning", "warning", _.toString(ans));
				return 'Error';
			}
		}
	}

	if (mode === "aimdfm") {
		$.QDfm.updateRow({
			table: "pdm_aimdfm_task",
			data: {
				progress: 33.33
			},
			where: { id: $.task_id }
		});
		if (GEN.hasError()) {
			Status = 'error';
			resultData.push({ type: "error", title: "GEN错误!", detail: [{ desc: _.join(GEN.STATUS, "\n") }] });
			return {
				status: Status,
				result_data: resultData
			};
		} else {
			resultData.push({ type: "info", title: "操作完成, 请注意检查!" });
			return {
				status: Status,
				result_data: resultData
			};
		}
	}else {
		// output and return status, if genesis error, it will output genesis error command
		if(GEN.STATUS.length == 0){
			return Return;
		}else{
			GUI.msgBox("error", "error", GEN.STATUS);
			return 'Error';
		}
	}
}catch (e) {	
	IKM.msg(_.join(GEN.STATUS, "\n"))
	IKM.msg(e)
	Status = 'error';
	resultData.push({type: "error", title: "脚本执行出错!", detail: [{desc: _.toString(e)}]});
	return (mode === "aimdfm") ? {status: Status, result_data: resultData} : "Error";
}

function save_profile_info(par){
	var step_name = par.step_type == 'array' ? 'array' : par.step;

	// ##
	var data = {
		job_id: JobId,
		step_name: step_name,
		step_type: par.step_type,
	}

	// ##
	var datum = GEN.getDatum({job:Job, step:par.step, units:PAR.units})
	data['datum_x'] = Math.round(Number(datum['x']) * 100) / 100;
	data['datum_y'] = Math.round(Number(datum['y']) * 100) / 100;

	// ##
	var profile = GEN.getProfile({job:Job,step:par.step,units:PAR.units})
	if(profile){
		data['profile'] = '';
		profile.split('\n').forEach(function(row){
			if(/^#/.test(row) && row.indexOf('Step profile data') == -1){
				data['profile'] += (row.replace(/#/g, '') + '\n');
			};
		})
		data['profile'] += 'SE';
	}

	// ##
	var profile_limits = GEN.getProfileLimits({job:Job,step:par.step,units:PAR.units});
	data['size_x'] = Math.round(Number(profile_limits['xsize']) * 100) / 100;
	data['size_y'] = Math.round(Number(profile_limits['ysize']) * 100) / 100;
	data['profile_limits'] = JSON.stringify(profile_limits);

	var array_size_x = data['size_x'];  // array尺寸
	var array_size_y = data['size_y'];

	// ##
	var sr_info = {};
	var sr = GEN.getSR1({job:Job,step:par.step,units:PAR.units});
	
	// print('=======>sr1:', _.toString(sr))

	var num = 0;
	sr.forEach(function(sr){
		sr['mirror'] = sr['mirror'] == 'yes' ? 1 : 0;
		sr['x'] = sr['xa'];
		sr['y'] = sr['ya'];
		sr_info[num] = sr;
		num++;
	})

	// print('=======>sr2:', _.toString(sr))

	if(Object.keys(sr_info).length > 0){
		data['step_repeat'] = JSON.stringify(sr_info)
	}

	data['sys_tags'] = '{import_from_cam}';

	// 存储array的尺寸
	if(par.step_type == 'array'){
		IKM.save_job_info({
			'jobid':JobId,
			'jobinfohash':{
				array_size_x: array_size_x.toFixed(2),
				array_size_y: array_size_y.toFixed(2),
			}
		})
	}

	// 存储信息
	IKM.db.query("", function(q) {
		var res;

		q.begin();
		try{
			res =  q.replaceRow({
				table: 'pdm_job_panelizer_step',
				data: data,
				update_policy: {tags:'array_append', attr_data:'json_merge'},
				unique_field: ['job_id', 'step_name'],
			});
			if (q.lastError().isValid()){
				throw q.lastError().text();
			}
			q.commit();
		}catch(e){
			q.rollback()
		}

		return res;
	});
	
	return null;
}

function UPLOAD_LAYER_MATRIX(props){   // 接收一个job名称  jobcategory这个参数不用管 默认work
    var job = props.job
    // props.jobcategory = props.jobcategory || "work"
    var matrix = ANALYSIS_STACKUP({job:job, jobcategory:props.jobcategory});  // 分析matrix数据
    var layers = Object.keys(matrix).sort(function(a,b){return matrix[a]["row"] - matrix[b]["row"]})
    var tableName = "pdm_job_" + props.jobcategory + "_layer";
    console.log("===========>matrix upload");
    var oLayers = IKM.db.query("",function(q){
        return q.selectMapMap({
            table:tableName,
            field: ["odb_name", "release_status"],
            where:{job_id:JobId},
            uniquefield: "odb_name"
        })
    });
    IKM.db.query("",function(q){
        return q.deleteRow({
            table:tableName,
            where:{job_id:JobId},
        })
    });
    var fieldLst = ["name", "odb_context", "odb_type", "odb_polarity", "odb_side", "drl_start", "drl_end",
        "row_num", "type", "drl_start_num", "drl_end_num", "drl_from_num", "drl_to_num", "drl_connect_to",
        "odb_name", "side", "stackup_num", "customer_field", "input_file_name", "odb_row_num"
    ];
    layers.forEach(function(layer){
        var layerInfo = matrix[layer];
        var tmpData = {
            "job_id": JobId,
            "name": layerInfo["name"]
        };
        tmpData["release_status"] = (oLayers&&oLayers.hasOwnProperty(layerInfo["odb_name"])) ? oLayers[layerInfo["odb_name"]]["release_status"] : null;
        for (n = 0; n < fieldLst.length; n++) {
            tmpData[fieldLst[n]] = layerInfo[fieldLst[n]];
        }
        IKM.db.query("",function(q){
            return q.insertRow({
                table:tableName,
                data: tmpData
            })
        });
    })
    console.log("===========>matrix upload end");
	
    return matrix
}

function ANALYSIS_STACKUP(props){  // 分析matrix数据
    var job = props.job
    // if(!props.hasOwnProperty("jobcategory")){ props.jobcategory = 'work' }
    var matrix = GEN.getMatrix({job:job})
    var layer_count = GEN.getLayerCount({job:job}) 
	IKM.save_job_info({jobid:JobId, jobcategory:props.jobcategory, jobinfohash:{TL_layer_count:layer_count}});
	_.values(matrix).sort(function(a,b){return a.row-b.row}).forEach(function(layer){
        layer.odb_name = layer.name;
		layer.name = layer.tl_name;
		layer.odb_context = layer.context;
		layer.odb_type = layer.layer_type;
		layer.odb_polarity = layer.polarity;
		layer.odb_side = layer.side;
		layer.row_num = layer.tl_num;
		layer.type = layer.tl_type;
		layer.side = layer.enum_tl_side;
		layer.input_file_name = layer.file_name;
		layer.odb_row_num = layer.row;

		if (layer.odb_name == 'drl'){
			layer.name = 'drl';
			layer.drl_start_num = 1;
			layer.drl_end_num = layer_count;
			layer.drl_from_num = 1;
			layer.drl_to_num = layer_count;
			layer.type = 'main_drill';
		}
		
		if(!layer.name){
			layer.name = '__'+layer.odb_name+'__'
		}
		if(!layer.type){
			layer.type = 'other'
		}
    })
    return matrix;
}
